@@ -18,6 +18,9 @@ from simple_report.xlsx.cursor import CalculateNextCursorXLSX
 
 __author__ = 'prefer'
 
+XLSX_GROUPING_COUNT = 30
+
+
 class SheetData(object):
     u"""
     self.read_data:
@@ -52,8 +55,8 @@ class SheetData(object):
 
     PREFIX_TAG = '%'
 
-    FIND_PARAMS = re.compile('#\w+#')
-    FIND_TEMPLATE_TAGS = re.compile('#{0}\w+{0}#'.format(PREFIX_TAG))
+    FIND_PARAMS = re.compile(u'#[A-zА-яёЁ_0-9]+#')
+    FIND_TEMPLATE_TAGS = re.compile(u'#{0}[A-zА-яёЁ_0-9]+{0}#'.format(PREFIX_TAG))
 
     COLUMN_INDEX = re.compile('[A-Z]+')
     ROW_INDEX = re.compile('[1-9][0-9]*')
@@ -98,6 +101,7 @@ class SheetData(object):
 
         # Ссылка на тег данных строк и столбцов листа с очищенными значениями
         self.write_data = self._write_xml.find(QName(self.ns, 'sheetData'))
+        self.write_data_dict = {}
         if not self.write_data is None:
             self.write_data.clear()
 
@@ -291,8 +295,21 @@ class SheetData(object):
         return cell.find(QName(self.ns, 'f'))
 
     def _create_or_get_output_row(self, row_index, attrib_row):
+        # найдем существующую строку в результирующих данных
+        key = self.XPATH_TEMPLATE_ROW % row_index
+        row_el = self.write_data_dict.get(key)
+
+        if row_el is None:
+            # если не нашли - создадим
+            row_el = SubElement(self.write_data, 'row', attrib=attrib_row)
+            self.write_data_dict[key] = row_el
+        return row_el
+
+
+    def _create_or_get_output_row_old(self, row_index, attrib_row):
         # найдем существующую строку в результирующих данных
         row_el = self.write_data.find(self.XPATH_TEMPLATE_ROW % row_index)
+
         if row_el is None:
             # если не нашли - создадим
             row_el = SubElement(self.write_data, 'row', attrib=attrib_row)
@@ -314,7 +331,7 @@ class SheetData(object):
     #     if self.calc_chain:
     #         calc = SubElement(self.calc_chain._root, 'c')
     #         calc.attrib['r'] = '%s%s' % (column, row)
-
+    #@profile
     def set_section(self, begin, end, start_cell, params, used_formulas=None):
         """
 
@@ -329,7 +346,7 @@ class SheetData(object):
 
             row_index = start_row + i
             attrib_row['r'] = str(row_index)
-
+            # следующая строка создает линейное время для флаша от числа флашей
             row_el = self._create_or_get_output_row(row_index, attrib_row)
 
             for j, col, cell in self._find_cells(range_cols, num_row, row):
@@ -354,7 +371,7 @@ class SheetData(object):
                     continue
 
                 value = self._get_tag_value(cell)
-                if not value is  None:
+                if not value is None:
                     value_el = SubElement(cell_el, 'v')
 
                     if attrib_cell.get('t') in ('n', None): # number
@@ -367,7 +384,6 @@ class SheetData(object):
                         value_string = self.shared_table.get_value(index_value)
 
                         who_found_params = self._get_values_by_re(value_string)
-
                         is_int = False
                         if who_found_params:
                             for found_param in who_found_params:
@@ -376,8 +392,13 @@ class SheetData(object):
                                 param_value = params.get(param_name)
                                 if used_formulas:
                                     formula_id_list = used_formulas.get(param_name)
-                                    assert (formula_id_list is None or
-                                    isinstance(formula_id_list, (list, tuple))), "used_formulas values must be lists or tuples"
+                                    assert (
+                                        formula_id_list is None or
+                                        isinstance(
+                                            formula_id_list,
+                                            (list, tuple))
+                                    ), ("used_formulas values must be "
+                                        "lists or tuples")
                                     if formula_id_list is not None:
                                         for formula_id in formula_id_list:
                                             cell_string = ''.join([col_index,
@@ -397,14 +418,20 @@ class SheetData(object):
                                         cell_el.attrib['t'] = 'n' # type - number
                                         value_el.text = unicode(days)
                                     else:
-                                        date_less_1900 = '%s.%s.%s' % (param_value.date().day,
-                                                                       param_value.date().month, param_value.date().year
-                                                                       ,)
+                                        date_less_1900 = '%s.%s.%s' % (
+                                            param_value.date().day,
+                                            param_value.date().month,
+                                            param_value.date().year
+                                        )
                                         # strftime(param_value, locale.nl_langinfo(locale.D_FMT)) - неработает для 1900 и ниже
                                         value_string = value_string.replace(found_param, unicode(date_less_1900))
 
-
-                                elif isinstance(param_value, (int, float, Decimal)) and found_param == value_string:
+                                # Добавил long, возможно нужно использовать
+                                # общего предка numbers.Number
+                                elif isinstance(
+                                    param_value,
+                                    (int, float, long, Decimal)
+                                ) and found_param == value_string:
                                     # В первую очередь добавляем числовые значения
                                     is_int = True
 
@@ -436,9 +463,29 @@ class SheetData(object):
                                                 func_, ':'.join([f_cells[0],
                                                                 f_cells[-1]])
                                                 )
-                                        else:
+                                        elif f_cells:
+                                            # Группируем по 30 элементов
+                                            cells_number = int(
+                                                len(f_cells) / XLSX_GROUPING_COUNT
+                                            ) + 1
+                                            val_list = []
+                                            for x in range(cells_number):
+                                                sub_args = (
+                                                    ','.join(
+                                                        f_cells[x * XLSX_GROUPING_COUNT:(x + 1) * XLSX_GROUPING_COUNT]
+                                                    )
+                                                )
+                                                if sub_args:
+                                                    sub_val = '(%s)' % (
+                                                        sub_args
+                                                    )
+                                                    val_list.append(sub_val)
+
                                             formula_el.text = '%s(%s)' % (
-                                                func_, ','.join(f_cells))
+                                                func_,
+                                                ','.join(val_list)
+                                            )
+
                                         self.formula_id_dict[f_id] = []
                                     else:
                                         continue
